/*
 * MainOIDPanel.java
 *
 * Created on 2 June 2008, 10:35
 */
package nz.co.abrahams.asithappens.oid;

import java.awt.Component;
import java.net.UnknownHostException;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import nz.co.abrahams.asithappens.cartgraph.DataGraph;
import nz.co.abrahams.asithappens.cartgraph.TimeSeriesContext;
import nz.co.abrahams.asithappens.core.DBException;
import nz.co.abrahams.asithappens.snmputil.SNMPAccessType;
import nz.co.abrahams.asithappens.snmputil.SNMPException;
import nz.co.abrahams.asithappens.snmputil.SNMPType;
import nz.co.abrahams.asithappens.snmputil.SNMPTypeException;
import nz.co.abrahams.asithappens.storage.DataSets;
import nz.co.abrahams.asithappens.storage.Device;
import nz.co.abrahams.asithappens.uiutil.DeviceSelectorModel;
import nz.co.abrahams.asithappens.uiutil.DeviceSelectorPanel;
import nz.co.abrahams.asithappens.uiutil.ErrorHandler;

/**
 *
 * @author mark
 */
public class MainOIDPanel extends javax.swing.JPanel {

    /**
     * SNMP type combo items
     */
    protected static final String[] oidTypes = {"Integer32", "Gauge32", "Counter32", "Counter64"};

    /**
     * Set display types
     */
    protected static final String[] displayTypes = {"Fill", "Line"};

    /**
     * The name of the current template being edited
     */
    protected String templateName;

    /**
     * The OID editing table
     */
    protected JTable oidTable;

    /**
     * Custom OID Template list
     */
    protected JList templatesList;

    /**
     * Device selector
     */
    private DeviceSelectorPanel deviceSelectorPanel;

    /**
     * Creates new form MainOIDPanel
     */
    public MainOIDPanel() {
        initComponents();
        initComponentsFinish();
        initializeTable();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        graphButton = new javax.swing.JButton();
        unitsLabel = new javax.swing.JLabel();
        unitsField = new javax.swing.JTextField();
        storeDataCheckBox = new javax.swing.JCheckBox();
        editPane = new javax.swing.JScrollPane();
        removeButton = new javax.swing.JButton();
        addButton = new javax.swing.JButton();
        pollLabel = new javax.swing.JLabel();
        pollField = new javax.swing.JTextField();
        pollUnitsLabel = new javax.swing.JLabel();

        setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        graphButton.setText("Create graph");
        graphButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphButtonActionPerformed(evt);
            }
        });
        add(graphButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 360, 550, 30));

        unitsLabel.setText("Y axis units");
        add(unitsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(410, 60, -1, -1));
        add(unitsField, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 60, 50, 20));

        storeDataCheckBox.setText("Store collected data");
        add(storeDataCheckBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 330, 180, 20));
        add(editPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 150, 550, 170));

        removeButton.setText("Remove OID");
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });
        add(removeButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(290, 100, 270, 40));

        addButton.setText("Add OID");
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        add(addButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, 270, 40));

        pollLabel.setText("Poll Interval");
        add(pollLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, -1, -1));

        pollField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        pollField.setText("2000");
        pollField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pollFieldActionPerformed(evt);
            }
        });
        add(pollField, new org.netbeans.lib.awtextra.AbsoluteConstraints(100, 60, 90, -1));

        pollUnitsLabel.setText("ms");
        add(pollUnitsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 60, -1, -1));
    }// </editor-fold>//GEN-END:initComponents

    private void initComponentsFinish() {
        deviceSelectorPanel = new DeviceSelectorPanel(SNMPAccessType.ReadOnly);
        add(deviceSelectorPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, 550, -1));
    }

    private void initializeTable() {
        JComboBox typeCombo;

        oidTable = new JTable(new CustomOIDTableModel());
        oidTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        editPane.setViewportView((Component) oidTable);

        typeCombo = new JComboBox();
        for (SNMPType type : SNMPType.values()) {
            typeCombo.addItem(type.toString());
        }
        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.Type.ordinal()).setCellEditor(new DefaultCellEditor(typeCombo));
        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.Type.ordinal()).setCellRenderer(new DefaultTableCellRenderer() {

            public void setValue(Object value) {
                setText((value == null) ? "" : ((SNMPType) value).label);
            }
        });

        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.Label.ordinal()).setPreferredWidth(140);
        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.OID.ordinal()).setPreferredWidth(220);
        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.Type.ordinal()).setPreferredWidth(120);
        oidTable.getColumnModel().getColumn(CustomOIDTableModel.Columns.Description.ordinal()).setPreferredWidth(220);
    }

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        ((CustomOIDTableModel) oidTable.getModel()).addOID();
    }//GEN-LAST:event_addButtonActionPerformed

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        if (oidTable.getSelectedRowCount() == 1) {
            ((CustomOIDTableModel) oidTable.getModel()).removeOID(oidTable.getSelectedRow());
        }
    }//GEN-LAST:event_removeButtonActionPerformed

    private void graphButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_graphButtonActionPerformed
        DeviceSelectorModel deviceSelector;
        String deviceName;
        Device device;
        int ifIndex;
        String portString;
        Vector<CustomOID> oids;
        CustomOIDSNMP snmp;
        CustomOIDCollectorDefinition definition;
        CustomOIDCollector collector;
        DataSets data;
        TimeSeriesContext context;
        DataGraph graphFrame;

        if (oidTable.getRowCount() == 0) {
            ErrorHandler.modalError(this, "Please add at least one OID to the table",
                    "No OIDs added");
            return;
        }


        deviceSelector = deviceSelectorPanel.getModel();
        deviceName = deviceSelector.getName();
        
        oids = ((CustomOIDTableModel) oidTable.getModel()).getCustomOIDVector();
        try {
            //device = new Device(deviceName, communityField.getText(), null, false);
            device = deviceSelector.loadDevice();
            snmp = new CustomOIDSNMP(device);
            definition = new CustomOIDCollectorDefinition(null, device,
                    Integer.parseInt(pollField.getText()), storeDataCheckBox.isSelected(),
                    oids, unitsField.getText());
            collector = new CustomOIDCollector(definition, snmp);
            data = new DataSets(collector);
            /*
            for (int i = 0; i < oids.size(); i++) {
                data.addSet(oids.elementAt(i).label);
            }
            */
            context = new TimeSeriesContext(data);
            graphFrame = new DataGraph(context);
        } catch (DBException e) {
            ErrorHandler.modalError(null, "Please ensure that database is running and accessible",
                    "Error opening database connection", e);
        } catch (UnknownHostException e) {
            ErrorHandler.modalError(null, "Please ensure that device name \"" + deviceName + "\" is valid",
                    "Unknown host " + deviceName, e);
        } catch (SNMPTypeException e) {
            ErrorHandler.modalError(null, "Please ensure that OID values and types are correct",
                    "Cannot access OID on " + deviceName, e);
        } catch (SNMPException e) {
            ErrorHandler.modalError(null, "Please ensure that device name and community string are correct",
                    "Cannot access SNMP service on device " + deviceName, e);
        }

    }//GEN-LAST:event_graphButtonActionPerformed

private void pollFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pollFieldActionPerformed
// TODO add your handling code here:
}//GEN-LAST:event_pollFieldActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addButton;
    private javax.swing.JScrollPane editPane;
    private javax.swing.JButton graphButton;
    private javax.swing.JTextField pollField;
    private javax.swing.JLabel pollLabel;
    private javax.swing.JLabel pollUnitsLabel;
    private javax.swing.JButton removeButton;
    private javax.swing.JCheckBox storeDataCheckBox;
    private javax.swing.JTextField unitsField;
    private javax.swing.JLabel unitsLabel;
    // End of variables declaration//GEN-END:variables
}
